% David Legault 1070008
% Fichier: fpassb2.m
% Description: Fonction permettant de d'extraire une demi-bande B d'un spectre numrique selon que l'on veut la
% dem-bande gauche ou droite et que l'on veut que cette demi-bande soit plac a gauche ou a droite de la frquence de
% rfrence f = 0
%
% Fonctions: fpassb(x, Nb, B)
%
% Variables:
%             x = spectre du signal a filtrer
%             Nb = Nombre de points dfinissant la largeur d'une bande
%             B = numro de la bande a isoler (B = 0 est la bande de base a f = 0)
%             L = demi-bande gauche (-1) ou droite (1)
%             S = shifting de la bande  gauche (-1) et droite (1)


function y = fpassb2(x,Nb,B,L,S)

N = length(x);

% extraction de la bande complete B
h = x(1,N/2+(2*B-1)*Nb/2+1:N/2+(2*B+1)*Nb/2);

Nh = length(h);

% on veut la demi-bande de gauche
if L == -1

  % on limine la demi-bande de droite
  h(1,Nh/2+1:Nh) = 0;
  
  % on veut que cette bande soit a droite, alors on shift le vecteur pour inverser les 2 parties [G, D] -> [D, G]
  if S == 1
    h = fftshift(h);
  end
  %z = z(1,1:Nz/2);
  
% on veut la bande de droite du spectre
else
  
  % toute la partie gauche est lmine
  h(1,1:Nh/2) = 0;
  
  % on veut que cette bande soit a gauche, alors on shift le vecteur pour inverser les 2 parties [G, D] -> [D, G]
  if S == -1
    h = fftshift(h);
  end
  %z = z(1,Nz/2+1:Nz);
end

% on garde que la bande non-nulle du spectre
if S == 1
  h = h(1,Nh/2+1:Nh);
else
  h = h(1,1:Nh/2);
end
  
y = h;