% David Legault 1070008
% Fichier: reconsm.m
% Description: Fonction qui recr un signal NRZ bas sur les valeurs chantillonnes du signal dans le cas
%              d'une transmission multi-usagers
%
% Fonctions: reconsm(x,t,pskz,pskz2,dt,N,Nk,T0,z,z2)
%
% Variables:
%             x = signal filtr
%             t = vecteur temps
%             pskz = vecteur des coefficients du signal z1
%             pskz2 = vecteur des coefficients du signal z2
%             dt = priode entre les chantillons
%             N = longueur du vecteur x
%             Nk = nombre de sous-bandes de coefficient dans x
%             T0 = priode du filtre
%             z = signal original z1
%             z2 = signal original z2

function y = reconsm(x,t,pskz,pskz2,dt,N,Nk,T0,z,z2)

%w_spectre(x,dt,42,'Spectre total bruit+signal');

y = fftshift(fft(x));
fm  = 0:1/(dt*N):1/dt - 1/(dt*N);
fm = fm - fm(ceil(length(fm)/2) + 1);
%t_graph(fm,y,70,'Rponse fft X Total');

% bande 0
B0 = fpassb3(y,dt*N,0);
fn = 1:1:length(B0);
%t_graph(fn,B0,71,'Bande 0');

% bande 1
B1 = fpassb3(y,dt*N,1);
%t_graph(fn,B1,72,'Bande 1');

% Coef PSI_0
PSI_0 = fpassb3(pskz(Nk/2+1,:),dt*N,0);
%t_graph(fn,PSI_0,73,'PSK_0');

% Coef PSI_1
PSI_1 = fpassb3(pskz(Nk/2+1+1,:),dt*N,0);
%t_graph(fn,PSI_1,74,'PSK_1');

Z1 = B0./(2*PSI_0) + B1./(2*PSI_1);
Z1 = [zeros(1,N/2-(dt*N)/2) Z1 zeros(1,N/2 - (dt*N)/2)];
%t_graph(fm,Z1,75,'Z1');

Z2 = B0./(2*PSI_0) - B1./(2*PSI_1);
Z2 = [zeros(1,N/2-(dt*N)/2) Z2 zeros(1,N/2 - (dt*N)/2)];
%t_graph(fm,Z2,76,'Z2');

zr1 = ifft(ifftshift(Z1));
zr2 = ifft(ifftshift(Z2));

figure(77);
plot(t,t_nrz(z),t,zr1,'--k','LineWidth',2);
xlim([min(t) max(t)]);
title('z1 reconstruit');
legend('reconstruit','original');
figure(78);
plot(t,t_nrz(z2),t,zr2,'--k','LineWidth',2);
xlim([min(t) max(t)]);
title('z2 reconstruit');
legend('original','reconstruit');

