% David Legault 1070008
% huffman.m
% Description :
%               Fonction de calcul de l'arbre de huffman de cardinalit D ( > 1). La fonction retourne h un vecteur
%               colonne contenant les mots de codes associes aux probabilits d'entre p (vecteur colonne). Elle
%               retourne galement la longueur moyenne l du code.
%
% Variables :
%               D = cardinalit de l'arbre
%               p = vecteur colonne des probabilits (il faut que ce vecteur soit class par ordre croissant de probabilit)

function [h,l]=huffman(p,D);

% calcul du nombre d'itrations de somme de D probabilits
ni = length(p) - D + 1;
% au moins une itration
if ni <= 0
  ni = 1;
end

% calcul des probabilits fantomes p = 0 a ajouter pour que le code soit optimal
c = mod(1-length(p),D-1);
% nouveau vecteur de probabilits
p = [zeros(c,1); p];
% nombre de probabilits a calculer
n = length(p);

% cration du vecteur complmentaire contenant des nombres puissance de 2 qui serviront d'indices associatifs pour les
% probabilits de p
no = 0:1:n-1;
o = (2.^no)';

% dfinition du vecteur de calcul contenant les probabilits et les indices de celles-ci
q = [p o];

% vecteur indicatif si une probabilit a dja complt son mot de code
o = [o zeros(n,1)];

% matrice d'indices de probabilits
ind = [];

% gnration de la matrice des indices associatifs
for i=1:ni
  q = sortrows(q,1);
  ind = [ind q(:,2)];
  q = [sum(q(1:D,:),1);q(D+1:n,:); ones(D-1,1) zeros(D-1,1)]; 
end

% cration du vecteur texte ncessaire pour les mots de code
for i=1:n
  h(i,:) = blanks(n-1);
end

% boucle de lecture globale de la matrice ind pour attribuer les symboles aux mot de codes
% boucle sur les colonnes en dbutant par la derniere
for j = ni:-1:1
  
  % symbole de dpart, valeurs 0 a D-1
  s = 0;
  
  % boucle sur chacune des lignes en arretant au dernier lment
  for i=1:n-(D-1)*(j-1)
    a = ind(i,j);
    
    % si on est en prsence d'un indice puissance de 2, alors son mot de code est termin
    if mod(a,2) == 0
      
      % condition de non rptition d'un indice dja vu auparavant
      if o(log2(a)+1,2) ~= 1
        % on attribue le symbole s a la probabilit associ a la position dans le vecteur des mots de code
        h(log2(a)+1,n-j) = num2str(s);
        % on indique que ce mot est complet
        o(log2(a)+1,2) = 1;
      end
    
    % sinon nous sommes en prsence d'un indice compos de plusieurs probabilits
    else
      % on vrifie lesquelles des probabilits sont prsentes dans l'indice et on lui attribue le symbole courant
      for k = 1:n
        if bitor(o(k,1),a) == a
          % on attribue le symbole s a la probabilit associ a la position dans le vecteur des mots de code
          h(log2(o(k,1))+1,n-j) = num2str(s);
        end
      end
    end
    % changement de symbole
    s = s + 1;
  end
end

% calcul de la longueur de chacun des mots de code
for i=1:n
  len(i) = length(deblank(h(i,:))); 
end

% calcul de la longueur moyenne du code
l = sum(p'.*len);